# auto-generated file
import _cffi_backend

ffi = _cffi_backend.FFI('sourmash._lowlevel__ffi',
    _version = 0x2601,
    _types = b'\x00\x00\xB5\x0D\x00\x00\xB8\x03\x00\x00\x16\x03\x00\x00\x00\x0F\x00\x00\x3D\x0D\x00\x00\x01\x11\x00\x00\x00\x0F\x00\x00\x3D\x0D\x00\x00\x16\x01\x00\x00\x16\x01\x00\x00\x01\x01\x00\x00\x01\x01\x00\x00\x01\x01\x00\x00\x18\x01\x00\x00\x18\x01\x00\x00\x01\x01\x00\x00\x00\x0F\x00\x00\x40\x0D\x00\x00\x4B\x03\x00\x00\x00\x0F\x00\x00\x40\x0D\x00\x00\x12\x11\x00\x00\x1A\x01\x00\x00\x00\x0F\x00\x00\x40\x0D\x00\x00\x1A\x01\x00\x00\x1A\x01\x00\x00\x1A\x01\x00\x00\x00\x0F\x00\x00\x40\x0D\x00\x00\x00\x0F\x00\x00\x31\x0D\x00\x00\x12\x11\x00\x00\x01\x01\x00\x00\x1A\x01\x00\x00\x12\x11\x00\x00\x02\x11\x00\x00\x00\x0F\x00\x00\x31\x0D\x00\x00\x12\x11\x00\x00\x1A\x01\x00\x00\x01\x01\x00\x00\x1A\x01\x00\x00\x12\x11\x00\x00\x02\x11\x00\x00\x00\x0F\x00\x00\x01\x0D\x00\x00\x00\x0F\x00\x00\xB9\x0D\x00\x00\x01\x03\x00\x00\x1A\x01\x00\x00\x00\x0F\x00\x00\xB9\x0D\x00\x00\x01\x11\x00\x00\x00\x0F\x00\x00\xB9\x0D\x00\x00\x12\x11\x00\x00\x00\x0F\x00\x00\xB9\x0D\x00\x00\x00\x0F\x00\x00\x0A\x0D\x00\x00\xB6\x03\x00\x00\x00\x0F\x00\x00\x0A\x0D\x00\x00\xB7\x03\x00\x00\x18\x01\x00\x00\x00\x0F\x00\x00\x0A\x0D\x00\x00\x01\x11\x00\x00\x01\x11\x00\x00\x00\x0F\x00\x00\x4B\x0D\x00\x00\x12\x11\x00\x00\x00\x0F\x00\x00\x4B\x0D\x00\x00\x02\x01\x00\x00\x00\x0F\x00\x00\xBB\x0D\x00\x00\x3D\x11\x00\x00\xB6\x03\x00\x00\x00\x0F\x00\x00\xBB\x0D\x00\x00\x40\x11\x00\x00\x00\x0F\x00\x00\x08\x0D\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\x08\x0D\x00\x00\x00\x0F\x00\x00\x8F\x0D\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\x0D\x0D\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\x0D\x0D\x00\x00\x3D\x11\x00\x00\x4F\x11\x00\x00\x00\x0F\x00\x00\x0D\x0D\x00\x00\x3D\x11\x00\x00\x18\x01\x00\x00\x00\x0F\x00\x00\x0D\x0D\x00\x00\x12\x11\x00\x00\x18\x01\x00\x00\x00\x0F\x00\x00\x16\x0D\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\x16\x0D\x00\x00\x40\x11\x00\x00\x00\x0F\x00\x00\x16\x0D\x00\x00\x40\x11\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\x16\x0D\x00\x00\x40\x11\x00\x00\x18\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x4F\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x12\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x12\x11\x00\x00\x01\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x16\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x0D\x03\x00\x00\x1A\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x3D\x11\x00\x00\x18\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x40\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x40\x11\x00\x00\x40\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x40\x11\x00\x00\x12\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x01\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x01\x11\x00\x00\x4F\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x01\x11\x00\x00\x12\x11\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\xB9\x03\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x0D\x03\x00\x00\x1A\x01\x00\x00\x00\x0F\x00\x00\xBE\x0D\x00\x00\x00\x0F\x00\x00\x3D\x03\x00\x00\x01\x09\x00\x00\x02\x09\x00\x00\x03\x09\x00\x00\x00\x09\x00\x00\x4B\x03\x00\x00\x0E\x01\x00\x00\x00\x0B\x00\x00\x01\x0B\x00\x00\x00\x01',
    _globals = (b'\xFF\xFF\xFF\x0BHASH_FUNCTIONS_MURMUR64_DAYHOFF',3,b'\xFF\xFF\xFF\x0BHASH_FUNCTIONS_MURMUR64_DNA',1,b'\xFF\xFF\xFF\x0BHASH_FUNCTIONS_MURMUR64_HP',4,b'\xFF\xFF\xFF\x0BHASH_FUNCTIONS_MURMUR64_PROTEIN',2,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_INTERNAL',2,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_INVALID_CODON_LENGTH',1103,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_INVALID_DNA',1101,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_INVALID_PROT',1102,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_IO',100001,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MISMATCH_DNAPROT',102,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MISMATCH_KSIZES',101,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MISMATCH_MAX_HASH',103,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MISMATCH_SEED',104,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MISMATCH_SIGNATURE_TYPE',105,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_MSG',3,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_NON_EMPTY_MIN_HASH',106,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_NO_ERROR',0,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_PANIC',1,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_PARSE_INT',100003,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_SERDE_ERROR',100004,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_UNKNOWN',4,b'\xFF\xFF\xFF\x0BSOURMASH_ERROR_CODE_UTF8_ERROR',100002,b'\x00\x00\x67\x23hash_murmur',0,b'\x00\x00\x92\x23kmerminhash_abunds_push',0,b'\x00\x00\x7C\x23kmerminhash_add_from',0,b'\x00\x00\x92\x23kmerminhash_add_hash',0,b'\x00\x00\x84\x23kmerminhash_add_sequence',0,b'\x00\x00\x80\x23kmerminhash_add_word',0,b'\x00\x00\x4D\x23kmerminhash_compare',0,b'\x00\x00\x4D\x23kmerminhash_containment_ignore_maxhash',0,b'\x00\x00\x5F\x23kmerminhash_count_common',0,b'\x00\x00\x3C\x23kmerminhash_dayhoff',0,b'\x00\x00\x79\x23kmerminhash_disable_abundance',0,b'\x00\x00\x79\x23kmerminhash_enable_abundance',0,b'\x00\x00\x79\x23kmerminhash_free',0,b'\x00\x00\x63\x23kmerminhash_get_abund_idx',0,b'\x00\x00\x59\x23kmerminhash_get_abunds',0,b'\x00\x00\x6B\x23kmerminhash_get_abunds_size',0,b'\x00\x00\x63\x23kmerminhash_get_min_idx',0,b'\x00\x00\x59\x23kmerminhash_get_mins',0,b'\x00\x00\x6B\x23kmerminhash_get_mins_size',0,b'\x00\x00\x54\x23kmerminhash_hash_function',0,b'\x00\x00\x89\x23kmerminhash_hash_function_set',0,b'\x00\x00\x3C\x23kmerminhash_hp',0,b'\x00\x00\x5F\x23kmerminhash_intersection',0,b'\x00\x00\x3C\x23kmerminhash_is_protein',0,b'\x00\x00\x54\x23kmerminhash_ksize',0,b'\x00\x00\x5C\x23kmerminhash_max_hash',0,b'\x00\x00\x7C\x23kmerminhash_merge',0,b'\x00\x00\x92\x23kmerminhash_mins_push',0,b'\x00\x00\x07\x23kmerminhash_new',0,b'\x00\x00\x54\x23kmerminhash_num',0,b'\x00\x00\x92\x23kmerminhash_remove_hash',0,b'\x00\x00\x8D\x23kmerminhash_remove_many',0,b'\x00\x00\x5C\x23kmerminhash_seed',0,b'\x00\x00\xAF\x23kmerminhash_slice_free',0,b'\x00\x00\x3C\x23kmerminhash_track_abundance',0,b'\x00\x00\x3F\x23nodegraph_count',0,b'\x00\x00\x51\x23nodegraph_expected_collisions',0,b'\x00\x00\x96\x23nodegraph_free',0,b'\x00\x00\x14\x23nodegraph_from_buffer',0,b'\x00\x00\x11\x23nodegraph_from_path',0,b'\x00\x00\x75\x23nodegraph_get',0,b'\x00\x00\x6E\x23nodegraph_ksize',0,b'\x00\x00\x71\x23nodegraph_matches',0,b'\x00\x00\x1D\x23nodegraph_new',0,b'\x00\x00\x6E\x23nodegraph_noccupied',0,b'\x00\x00\x6E\x23nodegraph_ntables',0,b'\x00\x00\x9D\x23nodegraph_save',0,b'\x00\x00\x6E\x23nodegraph_tablesize',0,b'\x00\x00\x99\x23nodegraph_update',0,b'\x00\x00\x18\x23nodegraph_with_tables',0,b'\x00\x00\x43\x23signature_eq',0,b'\x00\x00\x04\x23signature_first_mh',0,b'\x00\x00\xA1\x23signature_free',0,b'\x00\x00\x34\x23signature_get_filename',0,b'\x00\x00\x34\x23signature_get_license',0,b'\x00\x00\x00\x23signature_get_mhs',0,b'\x00\x00\x34\x23signature_get_name',0,b'\x00\x00\x2E\x23signature_new',0,b'\x00\x00\xA4\x23signature_push_mh',0,b'\x00\x00\x34\x23signature_save_json',0,b'\x00\x00\xA8\x23signature_set_filename',0,b'\x00\x00\xA4\x23signature_set_mh',0,b'\x00\x00\xA8\x23signature_set_name',0,b'\x00\x00\x26\x23signatures_load_buffer',0,b'\x00\x00\x1F\x23signatures_load_path',0,b'\x00\x00\x30\x23signatures_save_buffer',0,b'\x00\x00\x4A\x23sourmash_aa_to_dayhoff',0,b'\x00\x00\x4A\x23sourmash_aa_to_hp',0,b'\x00\x00\xB3\x23sourmash_err_clear',0,b'\x00\x00\x3A\x23sourmash_err_get_backtrace',0,b'\x00\x00\x57\x23sourmash_err_get_last_code',0,b'\x00\x00\x3A\x23sourmash_err_get_last_message',0,b'\x00\x00\xB3\x23sourmash_init',0,b'\x00\x00\xAC\x23sourmash_str_free',0,b'\x00\x00\x37\x23sourmash_str_from_cstr',0,b'\x00\x00\x47\x23sourmash_translate_codon',0),
    _struct_unions = ((b'\x00\x00\x00\xB9\x00\x00\x00\x02$SourmashStr',b'\x00\x00\xBA\x11data',b'\x00\x00\x16\x11len',b'\x00\x00\x0A\x11owned'),(b'\x00\x00\x00\xB6\x00\x00\x00\x10KmerMinHash',),(b'\x00\x00\x00\xB7\x00\x00\x00\x10Nodegraph',),(b'\x00\x00\x00\xB8\x00\x00\x00\x10Signature',)),
    _enums = (b'\x00\x00\x00\xBC\x00\x00\x00\x16HashFunctions\x00HASH_FUNCTIONS_MURMUR64_DNA,HASH_FUNCTIONS_MURMUR64_PROTEIN,HASH_FUNCTIONS_MURMUR64_DAYHOFF,HASH_FUNCTIONS_MURMUR64_HP',b'\x00\x00\x00\xBD\x00\x00\x00\x16SourmashErrorCode\x00SOURMASH_ERROR_CODE_NO_ERROR,SOURMASH_ERROR_CODE_PANIC,SOURMASH_ERROR_CODE_INTERNAL,SOURMASH_ERROR_CODE_MSG,SOURMASH_ERROR_CODE_UNKNOWN,SOURMASH_ERROR_CODE_MISMATCH_KSIZES,SOURMASH_ERROR_CODE_MISMATCH_DNAPROT,SOURMASH_ERROR_CODE_MISMATCH_MAX_HASH,SOURMASH_ERROR_CODE_MISMATCH_SEED,SOURMASH_ERROR_CODE_MISMATCH_SIGNATURE_TYPE,SOURMASH_ERROR_CODE_NON_EMPTY_MIN_HASH,SOURMASH_ERROR_CODE_INVALID_DNA,SOURMASH_ERROR_CODE_INVALID_PROT,SOURMASH_ERROR_CODE_INVALID_CODON_LENGTH,SOURMASH_ERROR_CODE_IO,SOURMASH_ERROR_CODE_UTF8_ERROR,SOURMASH_ERROR_CODE_PARSE_INT,SOURMASH_ERROR_CODE_SERDE_ERROR'),
    _typenames = (b'\x00\x00\x00\x08HashFunctions',b'\x00\x00\x00\xB6KmerMinHash',b'\x00\x00\x00\xB7Nodegraph',b'\x00\x00\x00\xB8Signature',b'\x00\x00\x00\x08SourmashErrorCode',b'\x00\x00\x00\xB9SourmashStr'),
)
